@@ -9,14 +9,14 @@ import sys
 import traceback
 from asyncio import Lock
 from pprint import pprint
-from typing import List, AsyncIterator, Optional, Callable, Generic, TypeVar
+from typing import List, AsyncIterator, Optional, Callable, Generic, TypeVar, Dict
 
 import aiohttp  # pip install aiohttp
 
 from .channel import Channel
 from .chat import Chat
 from .device import Device
-from .errors import HttpError, PushbulletError
+from .errors import HttpError, PushbulletError, InvalidKeyError
 from .filetype import get_file_type
 from .pushbullet import Pushbullet
 from .subscription import Subscription
@@ -41,14 +41,14 @@ class PushbulletAsyncIterator(AsyncIterator, Generic[T]):
                  _modified_after: float = None,
                  _post_process: Callable = None):
         # Passed args
-        self._pb = parent_pb  # type: AsyncPushbullet
-        self._url = _url
-        self._item_name = _item_name
-        self._limit = _limit
-        self._page_size = _page_size
-        self._active_only = _active_only
-        self._modified_after = _modified_after
-        self._post_process = _post_process
+        self._pb: AsyncPushbullet = parent_pb
+        self._url: str = _url
+        self._item_name: str = _item_name
+        self._limit: int = _limit
+        self._page_size: int = _page_size
+        self._active_only: bool = _active_only
+        self._modified_after: float = _modified_after
+        self._post_process: Callable = _post_process
 
         # Parameters for HTTP calls
         self._params = {}
@@ -61,14 +61,14 @@ class PushbulletAsyncIterator(AsyncIterator, Generic[T]):
 
         # Internal management
         self.log = logging.getLogger(__name__ + "." + self.__class__.__name__)
-        self.loop = None  # type: asyncio.BaseEventLoop
-        self._paused = False  # type: bool
-        self._stop = None  # Instructed to stop or end of iterator
-        self._objects = []  # type: List  # Use as FIFO queue of objects retrieved
-        self._total_objects_returned = 0  # type: int # Count of objects actually returned by iterator
-        self._paused_lock = Lock()  # type: asyncio.Lock  # Used to coordinate pausing on asyncio event loop
-        self._get_more = True  # type: bool  # Flag meaning there's more data to retrieve
-        self._first_async_run = True  # type: bool  # First chance in an async function, do some housekeeping
+        self.loop: asyncio.BaseEventLoop = None
+        self._paused: bool = False
+        self._stop: bool = None  # Instructed to stop or end of iterator
+        self._objects: List[Dict] = []  # Use as FIFO queue of objects retrieved
+        self._total_objects_returned: int = 0  # Count of objects actually returned by iterator
+        self._paused_lock: asyncio.Lock = None  # Used to coordinate pausing on asyncio event loop
+        self._get_more: bool = True  # Flag meaning there's more data to retrieve
+        self._first_async_run: bool = True  # First chance in an async function, do some housekeeping
 
         # This is so weird.  When using tkinter, the Lock needs to be saved in a
         # class-level location -- not even object-level -- so that if the application
@@ -102,7 +102,13 @@ class PushbulletAsyncIterator(AsyncIterator, Generic[T]):
 
     @property
     def empty(self):
-        return not bool(self._objects)
+        return len(self._objects) == 0
+
+    @property
+    def paused_lock(self):
+        if self._paused_lock is None:
+            self._paused_lock = asyncio.Lock()
+        return self._paused_lock
 
     async def async_pause(self, val: bool = True):
         """Pauses or unpauses the iterator.
@@ -116,11 +122,11 @@ class PushbulletAsyncIterator(AsyncIterator, Generic[T]):
         """
         if val and not self._paused:
             self._paused = True
-            await self._paused_lock.acquire()
+            await self.paused_lock.acquire()
         elif not val:
             self._paused = False
-            if self._paused_lock.locked():
-                self._paused_lock.release()
+            if self.paused_lock.locked():
+                self.paused_lock.release()
 
     def pause(self, val: bool = True):
         """Pauses or unpauses the iterator.
@@ -150,17 +156,16 @@ class PushbulletAsyncIterator(AsyncIterator, Generic[T]):
         return self
 
     async def __anext__(self) -> T:
-
         if self._first_async_run:
             self.loop = asyncio.get_event_loop()
             await self._pb.async_verify_key()
             self._first_async_run = False
 
         async def _check_if_paused():
-            await self._paused_lock.acquire()
-            self._paused_lock.release()
+            await self.paused_lock.acquire()
+            self.paused_lock.release()
             if self.stopped:
-                raise StopAsyncIteration("PushbulletAsyncIterator was told to stop.")
+                raise StopAsyncIteration("PushbulletAsyncIterator has been told to stop.")
 
         try:
             await _check_if_paused()
@@ -178,9 +183,13 @@ class PushbulletAsyncIterator(AsyncIterator, Generic[T]):
                     # Do I/O
                     msg = await self._pb._async_get_data(self._url, params=self._params)
 
+                except InvalidKeyError as ike:
+                    raise ike  # Pass this one along
+
                 except PushbulletError as pe:
                     self.log.debug("An error aborted the network request for _objects_asynciter: {}".format(pe))
-                    raise StopAsyncIteration(pe).with_traceback(sys.exc_info()[2])
+                    # raise StopAsyncIteration(pe).with_traceback(sys.exc_info()[2])
+                    raise pe
 
                 else:
                     items_this_round = msg.get(self._item_name, [])
@@ -225,8 +234,6 @@ class PushbulletAsyncIterator(AsyncIterator, Generic[T]):
 
         except StopAsyncIteration as sai:
             self.log.debug("Reason for iterator stopping: {}".format(sai))
-            # print("Caught StopAsyncIteration", sai, flush=True)
-            # traceback.print_tb(sys.exc_info()[2])
             self._stop = sai
             raise sai
 
@@ -235,14 +242,14 @@ class AsyncPushbullet(Pushbullet):
     """Provides access to pushbullet.com services using asyncio."""
 
     # This is weird.  See the note in PushbulletAsyncIterator.
-    _iterator_locks = []  # type: List[asyncio.Lock]
+    # _iterator_locks: List[asyncio.Lock] = []
 
     def __init__(self, api_key: str = None, verify_ssl: bool = None, *kargs, **kwargs):
         Pushbullet.__init__(self, api_key, *kargs, **kwargs)
 
-        self.loop = None  # type: asyncio.BaseEventLoop
-        self._aio_session = None  # type: aiohttp.ClientSession
-        self.verify_ssl = verify_ssl
+        self.loop: asyncio.BaseEventLoop = None
+        self._aio_session: aiohttp.ClientSession = None
+        self.verify_ssl: bool = verify_ssl
 
     async def __aenter__(self):
         await self.async_verify_key()
@@ -267,14 +274,13 @@ class AsyncPushbullet(Pushbullet):
 
          Raises a PushbulletError if something goes wrong.
          """
-        session = self._aio_session
+        session: aiohttp.ClientSession = self._aio_session
 
         if session is None or session.closed:
-            self.log.info("Creating aiohttp-based, asyncio session.")
+            self.log.debug("Creating aiohttp-based, asyncio session.")
 
             self.loop = asyncio.get_event_loop()
 
-            # print("Session is None, creating new one")
             headers = {"Access-Token": self.api_key}
 
             aio_connector = None  # type: aiohttp.TCPConnector
@@ -298,11 +304,12 @@ class AsyncPushbullet(Pushbullet):
             except Exception as ex:
                 await session.close()
                 self._aio_session = None
-                if isinstance(ex, PushbulletError):
-                    raise ex
-                else:
-                    tb = sys.exc_info()[2]
-                    raise PushbulletError(ex).with_traceback(tb) from ex
+                raise ex
+                # if isinstance(ex, PushbulletError):
+                #     raise ex
+                # else:
+                #     tb = sys.exc_info()[2]
+                #     raise PushbulletError(ex).with_traceback(tb) from ex
 
         return session
 
@@ -328,28 +335,27 @@ class AsyncPushbullet(Pushbullet):
 
     async def _async_http(self, aiohttp_func, url: str, **kwargs) -> dict:
 
-        try:
-            async with aiohttp_func(url + "", proxy=self.proxy, **kwargs) as resp:  # Do HTTP
-                code = resp.status
-                msg = None
-                # noinspection PyBroadException
-                try:
-                    if code != 204:  # 204 would be "No content"
-                        msg = await resp.json()
-                except:
-                    pass
-                finally:
-                    if msg is None:
-                        msg = await resp.read()
+        # try:
+        async with aiohttp_func(url + "", proxy=self.proxy, **kwargs) as resp:  # Do HTTP
+            code = resp.status
+            msg = None
+            # noinspection PyBroadException
+            try:
+                if code != 204:  # 204 would be "No content"
+                    msg = await resp.json()
+            except:
+                pass
+            finally:
+                if msg is None:
+                    msg = await resp.read()
 
-                return self._interpret_response(code, resp.headers, msg)
+            return self._interpret_response(code, resp.headers, msg)
 
-        except Exception as ex:
-            if isinstance(ex, PushbulletError):
-                raise ex
-            else:
-                tb = sys.exc_info()[2]
-                raise PushbulletError(ex).with_traceback(tb) from ex
+    # except Exception as ex:
+    #     if isinstance(ex, PushbulletError):
+    #         raise ex
+    #     else:
+    #         raise PushbulletError(ex).with_traceback(sys.exc_info()[2]) from ex
 
     async def _async_get_data(self, url: str, **kwargs) -> dict:
         session = await self.aio_session()
@@ -460,7 +466,7 @@ class AsyncPushbullet(Pushbullet):
             if nickname:
                 return next((z for z in self._devices if getattr(z, "nickname") == nickname), None)
             elif iden:
-                return next((y for y in self._devices if y.device_iden == iden), None)
+                return next((y for y in self._devices if y.iden == iden), None)
 
         x = _get()
         if x is None:
@@ -485,11 +491,11 @@ class AsyncPushbullet(Pushbullet):
                                           manufacturer=manufacturer, icon=icon, has_sms=has_sms)
         xfer = next(gen)
         data = xfer["data"]
-        xfer["msg"] = await self._async_post_data("{}/{}".format(self.DEVICES_URL, device.device_iden), data=data)
+        xfer["msg"] = await self._async_post_data("{}/{}".format(self.DEVICES_URL, device.iden), data=data)
         return next(gen)
 
     async def async_remove_device(self, device: Device) -> dict:
-        data = await self._async_delete_data("{}/{}".format(self.DEVICES_URL, device.device_iden))
+        data = await self._async_delete_data("{}/{}".format(self.DEVICES_URL, device.iden))
         return data
 
     # ################
@@ -641,7 +647,7 @@ class AsyncPushbullet(Pushbullet):
         _ = await self.async_get_channels()  # If no cached copy, create one
 
         def _get():
-            return next((z for z in self._channels if z.channel_tag == channel_tag), None)
+            return next((z for z in self._channels if z.tag == channel_tag), None)
 
         x = _get()
         if x is None:
@@ -728,7 +734,7 @@ class AsyncPushbullet(Pushbullet):
 
         def _get():
             return next((z for z in self._subscriptions
-                         if z.channel and z.channel.channel_tag == channel_tag), None)
+                         if z.channel and z.channel.tag == channel_tag), None)
 
         x = _get()
         if x is None:
@@ -855,13 +861,17 @@ class AsyncPushbullet(Pushbullet):
         msg = await self._async_delete_data(self.PUSH_URL)
         return msg
 
-    async def async_push_note(self, title: str,
-                              body: str,
+    async def async_push_note(self, title: str = None,
+                              body: str = None,
                               device: Device = None,
                               chat: Chat = None,
                               email: str = None,
                               channel: Channel = None) -> dict:
-        data = {"type": "note", "title": title, "body": body}
+        data = {"type": "note"}  # , "title": title, "body": body}
+        if title:
+            data["title"] = str(title)
+        if body:
+            data["body"] = str(body)
         data.update(Pushbullet._recipient(device=device,
                                           chat=chat,
                                           email=email,
@@ -972,3 +982,44 @@ class AsyncPushbullet(Pushbullet):
         data = xfer.get("data")
         xfer["msg"] = await self._async_push(data)
         return next(gen)
+
+    # Transfer file sub-function
+    async def async_push_file_simple(self,
+                               file_path: str,
+                               use_transfer_sh: bool = True,
+                               quiet: bool = False,
+                               title: str = None,
+                               body: str = None,
+                               target_device: Device = None):
+        if not os.path.isfile(file_path):
+            # TODO: this error handler is from the command line version
+            print("File not found:", file_path, file=sys.stderr)
+            return
+
+        show_progress = not quiet
+        if use_transfer_sh:
+            if not quiet:
+                print("Uploading file to transfer.sh ... {}".format(file_path))
+
+            info: Dict = await self.async_upload_file_to_transfer_sh(file_path=file_path,
+                                                                     show_progress=show_progress)
+
+        else:
+            if not quiet:
+                print("Uploading file to Pushbullet ... {}".format(file_path))
+            info: Dict = await self.async_upload_file(file_path=file_path,
+                                                      show_progress=show_progress)
+
+        file_url: str = info["file_url"]
+        file_type: str = info["file_type"]
+        file_name: str = info["file_name"]
+        if not quiet:
+            print("Pushing file ... {}".format(file_url))
+
+        title = title or "File: {}".format(file_name)
+        await self.async_push_file(file_name=file_name,
+                                   file_type=file_type,
+                                   file_url=file_url,
+                                   title=title,
+                                   body=body or file_url,
+                                   device=target_device)
